<?php
/**
 * @package		Donations
 * @author		<a href='https://www.devfuse.com'>DevFuse</a>
 * @copyright	(c) 2021 DevFuse
 */

namespace IPS\donate\extensions\core\Queue;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Background Task
 */
class _membersLegacyField
{
    /**
     * @brief Number of content items to rebuild per cycle
     */
    public $rebuild	= \IPS\REBUILD_SLOW;

    /**
     * Parse data before queuing
     *
     * @param	array	$data
     * @return	array
     */
    public function preQueueData( $data )
    {
        /* Get all members who have donations */
        try
        {
            $data['count'] = (int) \IPS\Db::i()->select( 'COUNT(member_id)', 'core_members', array( "donate_donations>?", 0 ) )->first();
        }
        catch( \Exception $e )
        {
            throw new \OutOfRangeException;
        }

        if( $data['count'] == 0 )
        {
            return null;
        }

        $data['completed'] = 0;

        return $data;
    }

    /**
     * Run Background Task
     *
     * @param	mixed						$data	Data as it was passed to \IPS\Task::queue()
     * @param	int							$offset	Offset
     * @return	int							New offset
     * @throws	\IPS\Task\Queue\OutOfRangeException	Indicates offset doesn't exist and thus task is complete
     */
    public function run( $data, $offset )
    {
        $last = NULL;

        $select = \IPS\Db::i()->select( '*', 'core_members', array( "member_id>? AND donate_donations>?", $offset, 0 ), 'member_id ASC', array( 0, $this->rebuild ) );

        foreach( new \IPS\Patterns\ActiveRecordIterator( $select, 'IPS\Member' ) as $member )
        {
            /* Convert members donations */
            \IPS\Db::i()->insert( 'donate_members', array(
                'donate_member_id' => $member->member_id,
                'donate_donations' => $member->donate_donations,
                'donate_amount'    => $member->donate_amount
            ), TRUE );

            $last = $member->member_id;
            $data['completed']++;
        }

        if( $last === NULL )
        {
            throw new \IPS\Task\Queue\OutOfRangeException;
        }

        return $last;
    }

    /**
     * Get Progress
     *
     * @param	mixed					$data	Data as it was passed to \IPS\Task::queue()
     * @param	int						$offset	Offset
     * @return	array	Text explaning task and percentage complete
     */
    public function getProgress( $data, $offset )
    {
        return array( 'text' => \IPS\Member::loggedIn()->language()->addToStack( 'queue_donate_rebuilding_legacy_member_field' ), 'complete' => $data['count'] ? ( round( 100 / $data['count'] * $data['completed'], 2 ) ) : 100 );
    }

    /**
     * Perform post-completion processing
     *
     * @param	array	$data		Data returned from preQueueData
     * @param	bool	$processed	Was anything processed or not? If preQueueData returns NULL, this will be FALSE.
     * @return	void
     */
    public function postComplete( $data, $processed = TRUE )
    {
        /* Remove custom table alters */
        if( \IPS\Db::i()->checkForColumn( 'core_members', 'donate_donations' ) )
        {
            \IPS\Db::i()->dropColumn( 'core_members', array( 'donate_donations' ) );
        }
        if( \IPS\Db::i()->checkForColumn( 'core_members', 'donate_amount' ) )
        {
            \IPS\Db::i()->dropColumn( 'core_members', array( 'donate_amount' ) );
        }
    }
}